Package: iAR
Type: Package
Title: Irregularly Observed Autoregressive Models
Version: 1.3.4
Date: 2026-05-14
Authors@R: c(
    person("Elorrieta", "Felipe", email = "felipe.elorrieta@usach.cl", role = c("aut","cre")),
    person("Ojeda", "Cesar", email = "cesar.ojeda@correounivalle.edu.co", role = "aut"),
    person("Eyheramendy", "Susana", email = "susana.eyheramendy@gmail.com", role = "aut"),
    person("Palma", "Wilfredo", email = "wilfredo.palma@gmail.com", role = "aut"))
Maintainer: Elorrieta Felipe <felipe.elorrieta@usach.cl>
Description: Data sets, functions and scripts with examples to implement autoregressive models for irregularly observed time series. The models available in this package are the irregular autoregressive model (Eyheramendy et al.(2018) <doi:10.1093/mnras/sty2487>), the complex irregular autoregressive model (Elorrieta et al.(2019) <doi:10.1051/0004-6361/201935560>) and the bivariate irregular autoregressive model (Elorrieta et al.(2021) <doi:10.1093/mnras/stab1216>).
License: GPL-2
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.7),ggplot2,stats, Rdpack, S7, zoo
RdMacros: Rdpack
Suggests: arfima, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/felipeelorrieta
LinkingTo: Rcpp,RcppArmadillo
LazyData: true
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-05-16 15:50:25 UTC; felocarril
Author: Elorrieta Felipe [aut, cre],
  Ojeda Cesar [aut],
  Eyheramendy Susana [aut],
  Palma Wilfredo [aut]
Repository: CRAN
Date/Publication: 2026-05-16 17:00:03 UTC
Config/roxygen2/version: 7.3.3
Config/testthat/edition: 3
RoxygenNote: 7.3.3
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-16 21:30:35 UTC; unix
Archs: iAR.so.dSYM
