# Generated by roxygen2: do not edit by hand

export(BiAR)
export(CiAR)
export(fit)
export(forecast)
export(gentime)
export(harmonicfit)
export(iAR)
export(interpolation)
export(kalman)
export(loglik)
export(multidata)
export(pairingits)
export(phase)
export(plot)
export(plot_fit)
export(plot_forecast)
export(sim)
export(summary)
export(unidata)
export(utilities)
import(Rcpp)
import(S7)
import(ggplot2)
import(zoo)
importFrom(Rdpack,reprompt)
importFrom(stats,anova)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(zoo,plot.zoo)
importFrom(zoo,zoo)
useDynLib(iAR, .registration = TRUE)
