# iAR <img src="https://pacha.dev/LSTS/hexicon.svg" width=150 align="right" alt="sticker"/>

## Description

Data sets, functions and scripts with examples to implement autoregressive models for irregularly observed time series. The models available in this package are the irregular autoregressive (iAR) model [Eyheramendy et al.(2018)](https://doi.org/10.1093/mnras/sty2487), the complex irregular autoregressive (CiAR) model [Elorrieta et al.(2019)](https://doi.org/10.1051/0004-6361/201935560) and the bivariate irregular autoregressive (BiAR) model [Elorrieta et al.(2021)](https://doi.org/10.1093/mnras/stab1216).

## Installation

You can install the released version of iAR from Github with:

``` r
remotes::install_github("felipeelorrieta/iAR-package")
```

## Code of Conduct
  
Please note that the iAR project is released with a [Contributor Code of Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.
